@@ -38,6 +38,49 @@ Chrome <- R6Class(
   )
 )
 
+#' Remote Chrome process
+#'
+#' @description
+#' Remote Chrome process
+#'
+#' @export
+ChromeRemote <- R6Class(
+  "ChromeRemote",
+  inherit = Browser,
+  public = list(
+    #' @description Create a new ChromeRemote object.
+    #' @param host A string that is a valid IPv4 or IPv6 address. `"0.0.0.0"`
+    #' represents all IPv4 addresses and `"::/0"` represents all IPv6 addresses.
+    #' @param port A number or integer that indicates the server port.
+    initialize = function(host, port) {
+      private$host <- host
+      private$port <- port
+    },
+
+    #' @description Is the remote service alive?
+    is_alive = function() {
+      url <- sprintf("http://%s:%s/json/version", private$host, private$port)
+
+      tryCatch(
+        {
+          # If we can read info from the remote host, then it's alive
+          suppressWarnings(fromJSON(url))
+          TRUE
+        },
+        error = function(err) FALSE
+      )
+    },
+
+    #' @description chromote does not manage remote processes, so closing a
+    #'   remote Chrome browser does nothing. You can send a `Browser$close()`
+    #'   command if this is really something you want to do.
+    close = function() {
+      # chromote didn't start this process, so we won't kill it or close it.
+      invisible(TRUE)
+    }
+  )
+)
+
 #' Find path to Chrome or Chromium browser
 #'
 #' @description
@@ -98,7 +141,7 @@ find_chrome <- function() {
 }
 
 chrome_verify <- function(path = find_chrome()) {
-  if (is_windows()) {
+  if (is_windows() && basename(path) != "chrome-headless-shell.exe") {
     return(chrome_verify_windows())
   }
 
@@ -164,7 +207,8 @@ chrome_windows_version_powershell <- function(path) {
   }
 
   output <- trimws(output)
-  output[nzchar(output)][[1]]
+  output <- output[nzchar(output)]
+  if (length(output) > 0) output[[1]] else ""
 }
 
 chrome_windows_version_wmic <- function(path) {
@@ -490,24 +534,3 @@ launch_chrome_impl <- function(path, args, port) {
     port = port
   )
 }
-
-#' Remote Chrome process
-#'
-#' @description
-#' Remote Chrome process
-#'
-#' @export
-ChromeRemote <- R6Class(
-  "ChromeRemote",
-  inherit = Browser,
-  public = list(
-    #' @description Create a new ChromeRemote object.
-    #' @param host A string that is a valid IPv4 or IPv6 address. `"0.0.0.0"`
-    #' represents all IPv4 addresses and `"::/0"` represents all IPv6 addresses.
-    #' @param port A number or integer that indicates the server port.
-    initialize = function(host, port) {
-      private$host <- host
-      private$port <- port
-    }
-  )
-)
