@@ -121,6 +121,23 @@ gen_command_body <- function(method_name, params) {
       })
     }
 
+  # As of 2025-02-07, it's not possible to query CDP to determine if the value
+  # of `mobile` in the device metrics override, so we need to track its value
+  # through any calls to `Emulation.setDeviceMetricsOverride`.
+  track_device_override_mobile <-
+    if (identical(method_name, "Emulation.setDeviceMetricsOverride")) {
+      expr({
+        if (!!sym("deviceScaleFactor") > 0) {
+          private$pixel_ratio <- !!sym("deviceScaleFactor")
+        } else {
+          private$pixel_ratio <- NULL
+        }
+        private$is_mobile <- !!sym("mobile")
+      })
+    } else {
+      expr({}) # fmt: skip
+    }
+
   # Construct parameters for message
   param_list <- lapply(params, function(param) {
     as.symbol(param$name)
@@ -144,6 +161,8 @@ gen_command_body <- function(method_name, params) {
     # Check for missing non-optional args
     !!!check_missing_exprs
 
+    !!!track_device_override_mobile
+
     msg <- list(
       method = !!method_name,
       params = drop_nulls(list(!!!param_list))
